#!/bin/sh

# Taken from https://github.com/postalserver/postal/blob/main/docker/wait-for.sh

[ -n "${DEBUG}" ] && set -x

check_http() {
  wget -T 1 -S -q -O /dev/null --no-check-certificate "$1" 2>&1 | head -1 |
    grep -E '^\s*HTTP.+\s[234][0-9]{2}' >/dev/null 2>&1
  return $?
}

check_tcp() {
  HOST="$(echo "$1" | cut -d: -f1)"
  PORT="$(echo "$1" | cut -d: -f2)"
  if [ -z "${HOST}" -o -z "${PORT}" ]; then
    echo "TCP target ${1} is not in \"<host>:<port>\" format" >&2
    exit 2
  fi

  nc -z -w1 "${HOST}" "${PORT}" >/dev/null 2>&1
  return $?
}

wait_for() {
  TYPE="$1"
  URI="$2"
  TIMEOUT="${3:-120}"

  SECONDS=0
  while [ "${SECONDS}" -lt "${TIMEOUT}" ] && ! "check_${TYPE}" "${URI}"; do
    if [ "${SECONDS}" -lt "1" ]; then
      printf "Waiting for %s  ." "${URI}"
    else
      printf .
    fi
    SECONDS=$((SECONDS + 1))
    sleep 1
  done

  if [ "${SECONDS}" -lt "${TIMEOUT}" ]; then
    if [ "${SECONDS}" -gt "0" ]; then
      echo "  up!"
    fi
  else
    echo "  FAIL"
    echo "ERROR: unable to connect to: ${URI}" >&2
    exit 1
  fi
}

for WAIT in $(printenv | grep -E '^WAIT_FOR_TARGETS(_.*)?' | cut -d= -f 1); do
  if [ -n "${WAIT}" ]; then
    for URI in $(printenv ${WAIT} | sed -E 's/,/\n/g' | uniq); do
      if echo "${URI}" | grep -E '^https?://.*' >/dev/null 2>&1; then
        wait_for "http" "${URI}" "${WAIT_FOR_TIMEOUT}"
      else
        wait_for "tcp" "${URI}" "${WAIT_FOR_TIMEOUT}"
      fi
    done
  fi
done
